' audio.spin                                            

CON

CLOCK_FREQUENCY = 100_000_000
SAMPLE_FREQUENCY = 44_100
DELAY_CYCLES    = CLOCK_FREQUENCY / SAMPLE_FREQUENCY

AUDIO_BUFFER_WORDS = 256

PIN_SHUTDOWN_B                  = 16
PIN_RIGHT                       = 17
PIN_LEFT                        = 18
PIN_HEADPHONE_DETECT            = 19

VAR
byte cog

PUB Start(pParams)
  dira[PIN_SHUTDOWN_B]~~
  outa[PIN_SHUTDOWN_B]~~
  cog := cognew(@Main, pParams)

PUB Stop
  cogstop(cog)

PUB AreHeadphonesPresent
  return ina[PIN_HEADPHONE_DETECT] <> 0  

DAT
                        ORG $000

Main
                        mov     dira, kDirections

                        ' frqa = right channel
                        movs      ctra, #PIN_RIGHT                       
                        movi      ctra, #%00110_000
                        mov       frqa, kBiasScaled
                         
                        ' frqb = left channel
                        movs      ctrb, #PIN_LEFT                                       
                        movi      ctrb, #%00110_000
                        mov       frqb, kBiasScaled
                         
                        rdlong  pBuffer, par
                        mov     pWriteCount, par
                        add     pWriteCount, #4
                        mov     pReadCount, par
                        add     pReadCount, #8

                        mov     cycles, cnt
                        add     cycles, kDelayCycles

:loop
                        call    #GetSample
                        waitcnt cycles, kDelayCycles
                        mov     frqa, rightSample
                        mov     frqb, leftSample
                        jmp     #:loop

GetSample
                        rdlong  writeCount, pWriteCount
                        mov     leftSample, kBiasScaled
                        mov     rightSample, kBiasScaled
                        rdlong  readCount, pReadCount
                        cmp     readCount, writeCount   wz
              if_z      jmp     #:empty
                        mov     r0, readCount
                        and     r0, kReadCountMask
                        add     r0, pBuffer              
                        rdword  leftSample, r0
                        add     r0, #2
                        add     leftSample, kBias
                        rdword  rightSample, r0
                        add     rightSample, kBias
                        and     leftSample, kSampleMask
                        shl     leftSample, #16
                        and     rightSample, kSampleMask
                        shl     rightSample, #16
                        add     readCount, #4
                        wrlong  readCount, pReadCount
:empty                        
GetSample_ret
                        ret
                        
kRampAdjust             long $0001_0000
kZero                   long 0
kOne                    long 1
kBias                   long $8000
kBiasScaled             long $8000_0000
kSampleMask             long |<16 - 1
kReadCountMask          long $0000_03FF

kDirections             long |<PIN_RIGHT | |<PIN_LEFT

kDelayCycles            long DELAY_CYCLES

leftSample              res 1
rightSample             res 1
cycles                  res 1

pBuffer                 res 1
pWriteCount             res 1
pReadCount              res 1
readCount               res 1
writeCount              res 1

r0            res 1        