' RaindropButtons.spin
' There are 12 input buttons which are interfaced to the Propeller via 2 74165-type shift registers.
' Four bits are not used, bits 0, 1, 10, 11.  They are tied to ground at the shift register inputs.

CON

PIN_LOAD_B                      = 21                    ' parallel load, active low
PIN_CLOCK                       = 22                    ' shift on positive edge                    
PIN_SERIAL_IN                   = 23                    ' most significant of 16 bits

MASK_SOFT_RIGHT                 = %0000_0000_0000_0100
MASK_START                      = %0000_0000_0000_1000
MASK_C                          = %0000_0000_0001_0000
MASK_A                          = %0000_0000_0010_0000

MASK_B                          = %0000_0000_0100_0100
MASK_D                          = %0000_0000_1000_0000
MASK_SELECT                     = %0000_0001_0000_0000
MASK_SOFT_LEFT                  = %0000_0010_0000_0000

MASK_LEFT                       = %0001_0000_0000_0100
MASK_DOWN                       = %0010_0000_0000_0000
MASK_UP                         = %0100_0000_0000_0000
MASK_RIGHT                      = %1000_0000_0000_0000

MASK_ANY                        = %1111_0011_1111_1100

VAR

word currentState
word previousState
word positiveEdge     ' positive edges are presses when there was no press before
word negativeEdge     ' negative edges are a release

PUB Start
  dira[PIN_LOAD_B]~~
  dira[PIN_CLOCK]~~
  outa[PIN_LOAD_B]~~
  outa[PIN_CLOCK]~

  currentState := positiveEdge := negativeEdge := 0
 
PUB Read | input
  outa[PIN_LOAD_B]~
  outa[PIN_LOAD_B]~~

  previousState := currentState
  currentState := (ina >> PIN_SERIAL_IN) & 1

  repeat 15
    outa[PIN_CLOCK]~~
    outa[PIN_CLOCK]~
    currentState <<= 1
    currentState |= (ina >> PIN_SERIAL_IN) & 1
         
  currentState := !currentState
  currentState &= %1111_0011_1111_1100                  ' mask out the unused bits
  positiveEdge := !previousState &  currentState
  negativeEdge :=  previousState & !currentState

PUB GetCurrentState
  return currentState

PUB IsPressed(mask)
  return currentState & mask

PUB WasPressed(mask)
  return positiveEdge & mask

PUB WasReleased(mask)
  return negativeEdge & mask
    