'RaindropDisplay.spin

CON

MILLISECOND_DELAY = 100_000_000 / 1000

PIN_ENABLE_12V        = 13
PIN_RESET_B           = 12
PIN_DATA_NOT_COMMAND  = 11
PIN_CHIP_SELECT_B     = 10
PIN_WRITE_STROBE_B    = 9
PIN_READ_STROBE_B     = 8
PIN_DATA_BUS          = 0
PIN_D0      = PIN_DATA_BUS
PIN_D7      = PIN_DATA_BUS + 7

DISPLAY_WIDTH    = 128
DISPLAY_HEIGHT   = 128
DISPLAY_AREA     = DISPLAY_WIDTH * DISPLAY_HEIGHT

' commands
CMD_SET_COLUMN_ADDRESS          = $15                   ' start, end
CMD_SET_ROW_ADDRESS             = $75                   ' start, end
CMD_WRITE_RAM                   = $5C                   '
CMD_READ_RAM                    = $5D                   '
CMD_SET_REMAP_AND_COLOR_DEPTH   = $A0                   '
CMD_SET_DISPLAY_START_LINE      = $A1                   '
CMD_SET_DISPLAY_OFFSET          = $A2                   '
CMD_DISPLAY_PIXELS_OFF          = $A4
CMD_DISPLAY_PIXELS_ON           = $A5
CMD_DISPLAY_PIXELS_NORMAL       = $A6
CMD_DISPLAY_PIXELS_INVERSE      = $A7
CMD_SET_MASTER_CONFIGURATION    = $AD
CMD_SLEEP_MODE_ON               = $AE
CMD_SLEEP_MODE_OFF              = $AF
CMD_SET_POWER_SAVING_MODE       = $B0
CMD_SET_RESET_PRECHARGE_PERIOD  = $B1
CMD_SET_CLOCK_PARAMETERS        = $B3
CMD_SET_PWM_LOOKUP_TABLE        = $B8
CMD_RESET_PWM_LOOKUP_TABLE      = $B9
CMD_SET_PRECHARGE_VOLTAGE       = $BB
CMD_SET_VCOMH                   = $BE
CMD_SET_CONTRAST_CURRENT        = $C1
CMD_SET_MASTER_CONTRAST         = $C7
CMD_SET_MUX_RATIO               = $CA
CMD_NOP                         = $E3
CMD_SET_COMMAND_LOCK            = $FD

CMD_DRAW_LINE                   = $83
CMD_DRAW_RECTANGLE              = $84
CMD_DRAW_CIRCLE                 = $86
CMD_COPY_RECTANGLE              = $8A
CMD_DIM_RECTANGLE               = $8C
CMD_CLEAR_RECTANGLE             = $8E
CMD_SET_FILL_ENABLE             = $92
CMD_SET_HORIZONTAL_SCROLL       = $96
CMD_STOP_HORIZONTAL_SCROLL      = $9E
CMD_START_HORIZONTAL_SCROLL     = $9F

PUB Start

  Init

  ' Clear screen
  WriteCommand(CMD_CLEAR_RECTANGLE)
  WriteData(0)
  WriteData(0)
  WriteData(130)
  WriteData(130)
  Delay(100)

  EnableDisplayPower

PUB EnableDisplayPower
  outa[PIN_ENABLE_12V]~~

PUB DisableDisplayPower
  outa[PIN_ENABLE_12V]~

PRI Init

  dira[PIN_ENABLE_12V..PIN_D0]~~

  outa[PIN_ENABLE_12V]~
  outa[PIN_D7..PIN_D0]~
  outa[PIN_READ_STROBE_B]~~
  outa[PIN_DATA_NOT_COMMAND]~
  outa[PIN_WRITE_STROBE_B]~
  outa[PIN_CHIP_SELECT_B]~

  Reset

  WriteCommand(CMD_SET_REMAP_AND_COLOR_DEPTH)
  WriteData(%00_1_0_0_0_1_0)                                  
  WriteCommand(CMD_SET_DISPLAY_START_LINE)
  WriteData(0)
  WriteCommand(CMD_SET_DISPLAY_OFFSET)
  WriteData(0)
  WriteCommand(CMD_DISPLAY_PIXELS_NORMAL)
  WriteCommand(CMD_SET_MASTER_CONFIGURATION)
  WriteData($8F)                ' $8E also works (reset)
  WriteCommand(CMD_SET_POWER_SAVING_MODE)
  WriteData($05)  
  WriteCommand(CMD_SET_RESET_PRECHARGE_PERIOD)             
  WriteData($74)                ' $11 also works  
  WriteCommand(CMD_SET_CLOCK_PARAMETERS)
  WriteData($F0)  
  WriteCommand(CMD_SET_PRECHARGE_VOLTAGE)
  WriteData($1C)  
  WriteData($1C)
  WriteData($1C)  
  WriteCommand(CMD_SET_VCOMH)
  WriteData($1F)  
  WriteCommand(CMD_SET_CONTRAST_CURRENT)             
  WriteData($C0)  
  WriteData($80)
  WriteData($80)  
  WriteCommand(CMD_SET_MASTER_CONTRAST)
  WriteData($0F)  
  WriteCommand(CMD_SET_MUX_RATIO)
  WriteData(127)
  WriteCommand(CMD_SET_COLUMN_ADDRESS)
  WriteData(4)
  WriteData(131)
  WriteCommand(CMD_SET_ROW_ADDRESS)
  WriteData(0)
  WriteData(127)
  WriteCommand(CMD_SLEEP_MODE_OFF)

  Delay(120)

PRI Delay(milliseconds)

  repeat milliseconds
    waitcnt(cnt + MILLISECOND_DELAY)

PRI Reset

  outa[PIN_RESET_B]~
  Delay(100)
  outa[PIN_RESET_B]~~

PRI WriteCommand(command)

  outa[PIN_DATA_NOT_COMMAND]~
  outa[PIN_CHIP_SELECT_B]~
  outa[PIN_WRITE_STROBE_B]~
  outa &= !($FF << PIN_DATA_BUS)
  outa |= (command & $FF) << PIN_DATA_BUS
  outa[PIN_WRITE_STROBE_B]~~
  outa[PIN_CHIP_SELECT_B]~~
  outa[PIN_DATA_NOT_COMMAND]~~

PRI WriteData(data)

  outa[PIN_CHIP_SELECT_B]~
  outa[PIN_WRITE_STROBE_B]~
  outa &= !($FF << PIN_DATA_BUS)
  outa |= (data & $FF) << PIN_DATA_BUS
  outa[PIN_WRITE_STROBE_B]~~
  outa[PIN_CHIP_SELECT_B]~~
  