' RaindropDisplayFb.spin
' Frame-buffered display
' This object provides a framebuffer and writes its contents to the display buffer when the flag is set

OBJ

display : "RaindropDisplay"

CON
DISPLAY_AREA_IN_LONGS           = display#DISPLAY_AREA / 4

VAR

long frameBuffer[DISPLAY_AREA_IN_LONGS]  

long frameBufferAddress
long flag

PUB Start
  display.Start
  dira[display#PIN_RESET_B..display#PIN_D0]~
  frameBufferAddress := @frameBuffer
  cognew(@Main, @frameBufferAddress)

PUB SetFlag(a)
  flag := a

PUB GetFlag
  return(flag)
  
PUB EnableDisplayPower
  display.EnableDisplayPower

PUB DisableDisplayPower
  display.DisableDisplayPower  

PUB PrintLong(theLong, x, y, color)

  repeat 8
    PrintNybble((theLong >> 28), x, y, color)
    theLong <<= 4
    x += 16

PUB PrintWord(theWord, x, y, color)

  repeat 4
    PrintNybble((theWord >> 12), x, y, color)
    theWord <<= 4
    x += 16

PUB PrintByte(theByte, x, y, color)

  repeat 2
    PrintNybble((theByte >> 4), x, y, color)
    theByte <<= 4
    x += 16

PUB PrintNybble(nybble, x, y, color)
  nybble &= $F
  
  if nybble => 0 AND nybble =< 9
    DrawChar(nybble + "0", x, y, color)
  else
    DrawChar(nybble + "a" - $A, x, y, color) 

PUB PrintString(pString, x, y, color) | c

  c := byte[pString]
  repeat while c <> 0
    DrawChar(c, x, y, color)
    x += 16
    pString++
    c := byte[pString]
   
PUB DrawChar(char, x, y, color) | pDest, pSource, row, mask, p

  pDest := @frameBuffer + (y << 7) + x
  pSource := $8000 + ((char >> 1) << 7)
  
  repeat 32
    mask := (char & 1) + 1
    p := long[pSource]
    repeat 16
      if (p & mask) <> 0
        byte[pDest] := color
      else
        byte[pDest] := 0
      pDest++
      mask <<= 2
    pDest += constant(display#DISPLAY_WIDTH - 16)
    pSource += 4
   
PUB Box(x0, y0, x1, y1, color)

  Line(x0, y0, x0, y1, color)
  Line(x1, y0, x1, y1, color)
  Line(x0, y0, x1, y0, color)
  Line(x0, y1, x1, y1, color)

PUB Line(x0, y0, x1, y1, color) | pScreen, dx, dy, major, minor, majorStep, minorStep, error, errorAdjust, errorStep

  pScreen := @frameBuffer + y0*display#DISPLAY_WIDTH + x0

  dx := x1 - x0
  dy := y1 - y0

  if ||dx => ||dy
    major := ||dx
    minor := ||dy
    majorStep := 1
    if dx < 0
      majorStep := -1
    minorStep := display#DISPLAY_WIDTH
    if dy < 0
      minorStep := -display#DISPLAY_WIDTH
  else
    major := ||dy
    minor := ||dx
    majorStep := display#DISPLAY_WIDTH
    if dy < 0
      majorStep := -display#DISPLAY_WIDTH
    minorStep := 1
    if dx < 0
      minorStep := -1

  error := major
  errorAdjust := major << 1
  errorStep := minor << 1

  repeat major + 1
    byte[pScreen] := color
    pScreen += majorStep
    error += errorStep
    if error => errorAdjust
      error -= errorAdjust
      pScreen += minorStep

DAT
                        ORG $000
Main
                        mov     outa, kOutputs
                        mov     dira, kDirections
                        
                        mov     pFrameBuffer, par
                        rdlong  pFrameBuffer, pFrameBuffer
                        mov     pFlag, par
                        add     pFlag, #4

:loop
                        rdlong  r0, pFlag       wz
              if_z      jmp     #:loop

                        andn    outa, kDataNotCommandMask
                        andn    outa, kChipSelectBMask
                        andn    outa, kWriteStrobeBMask
                        andn    outa, kDataBusMask
                        or      outa, kCmdWriteRam
                        or      outa, kWriteStrobeBMask
                        or      outa, kChipSelectBMask
                        or      outa, kDataNotCommandMask

                        mov     r0, kDisplayArea
                        shr     r0, #2
:pixelLoop
                        rdbyte  pixel, pFrameBuffer
                        add     pFrameBuffer, #1
                        andn    outa, kPrepareDataWriteMask

                        rdbyte  pixel1, pFrameBuffer
                        add     pFrameBuffer, #1
                        or      outa, pixel

                        rdbyte  pixel2, pFrameBuffer
                        add     pFrameBuffer, #1
                        or      outa, kFinishDataWriteMask

                        rdbyte  pixel3, pFrameBuffer
                        add     pFrameBuffer, #1
                        andn    outa, kPrepareDataWriteMask

                        or      outa, pixel1
                        or      outa, kFinishDataWriteMask

                        andn    outa, kPrepareDataWriteMask
                        or      outa, pixel2
                        or      outa, kFinishDataWriteMask

                        andn    outa, kPrepareDataWriteMask
                        or      outa, pixel3
                        or      outa, kFinishDataWriteMask

                        djnz    r0, #:pixelLoop
                        sub     pFrameBuffer, kDisplayArea
                        jmp     #:loop                                                                                                                         
                        
kDisplayArea            long display#DISPLAY_AREA
kCmdWriteRam            long display#CMD_WRITE_RAM

kDirections             long %00000000_00000000_00011111_11111111
kOutputs                long %00000000_00000000_00011111_00000000

kDataNotCommandMask     long 1 << display#PIN_DATA_NOT_COMMAND
kChipSelectBMask        long 1 << display#PIN_CHIP_SELECT_B
kWriteStrobeBMask       long 1 << display#PIN_WRITE_STROBE_B
kDataBusMask            long $FF
kPrepareDataWriteMask   long (1 << display#PIN_CHIP_SELECT_B) | (1 << display#PIN_WRITE_STROBE_B) | ($FF)
kFinishDataWriteMask    long (1 << display#PIN_CHIP_SELECT_B) | (1 << display#PIN_WRITE_STROBE_B)

pFrameBuffer  res 1
pFlag         res 1
pixel         res 1
pixel1        res 1
pixel2        res 1
pixel3        res 1

r0            res 1